@@ -83,9 +83,10 @@
 # 
 ##############################################################################
 
-'''$Id: db.py 561 2009-01-05 23:52:13Z eikenberry $'''
-__version__='$Revision: 561 $'[11:-2]
+'''$Id: db.py 649 2010-11-10 02:27:54Z eikenberry $'''
+__version__='$Revision: 649 $'[11:-2]
 
+import time
 import _mysql
 import MySQLdb
 from _mysql_exceptions import (OperationalError, NotSupportedError,
@@ -484,7 +485,7 @@ class DB(joinTM):
             info['extra'] = Extra,
             info['description'] = ' '.join([Type, field_default, Extra or '',
                                         key_types.get(Key, Key or ''),
-                                        Null != 'NULL' and 'NOT NULL' or ''])
+                                        Null == 'NO' and 'NOT NULL' or ''])
             info['nullable'] = (Null == 'YES') and 1 or 0
             if Key:
                 info['index'] = True
@@ -634,32 +635,40 @@ class DB(joinTM):
         else:
             LOG.error("aborting when non-transactional")
 
+    def _mysql_version(self):
+        """ Return mysql server version.
+            Note instances of this class are not persistent.
+        """
+        _version = getattr(self, '_version', None)
+        if not _version:
+            self._version = _version = self.variables().get('version')
+        return _version
+
     def savepoint(self):
         """ Basic savepoint support.
 
             Raise AttributeErrors to trigger optimistic savepoint handling
             in zope's transaction code.
         """
-        if self.variables().get('version') < '5.0.2':
+        if self._mysql_version() < '5.0.2':
             # mysql supports savepoints in versions 5.0.3+
-            LOG.error("Savepoints unsupported with Mysql < 5.0.3")
+            LOG.warning("Savepoints unsupported with Mysql < 5.0.3")
             raise AttributeError
 
         if not self._transaction_begun:
             LOG.error("Savepoint used outside of transaction.")
             raise AttributeError
 
-        import time
-        return _SavePoint(self, str(time.time()).replace('.','sp'))
+        return _SavePoint(self)
 
 
 class _SavePoint(object):
     """ Simple savepoint object
     """
-    def __init__(self, dm, ident):
-        dm._query("SAVEPOINT %s" % ident)
+    def __init__(self, dm):
         self.dm = dm
-        self.ident = ident
+        self.ident = ident = str(time.time()).replace('.','sp')
+        dm._query("SAVEPOINT %s" % ident)
 
     def rollback(self):
         self.dm._query("ROLLBACK TO %s" % self.ident)
